import { Event } from '../../../../../base/common/event.js';

/**
 * A model of the prompt input state using shell integration and analyzing the terminal buffer. This
 * may not be 100% accurate but provides a best guess.
 */
interface IPromptInputModel extends IPromptInputModelState {
    readonly onDidStartInput: Event<IPromptInputModelState>;
    readonly onDidChangeInput: Event<IPromptInputModelState>;
    readonly onDidFinishInput: Event<IPromptInputModelState>;
    /**
     * Fires immediately before {@link onDidFinishInput} when a SIGINT/Ctrl+C/^C is detected.
     */
    readonly onDidInterrupt: Event<IPromptInputModelState>;
    /**
     * Gets the prompt input as a user-friendly string where `|` is the cursor position and `[` and
     * `]` wrap any ghost text.
     */
    getCombinedString(): string;
}
interface IPromptInputModelState {
    /**
     * The full prompt input include ghost text.
     */
    readonly value: string;
    /**
     * The prompt input up to the cursor index, this will always exclude the ghost text.
     */
    readonly prefix: string;
    /**
     * The prompt input from the cursor to the end, this _does not_ include ghost text.
     */
    readonly suffix: string;
    /**
     * The index of the cursor in {@link value}.
     */
    readonly cursorIndex: number;
    /**
     * The index of the start of ghost text in {@link value}. This is -1 when there is no ghost
     * text.
     */
    readonly ghostTextIndex: number;
}
interface ISerializedPromptInputModel {
    readonly modelState: IPromptInputModelState;
    readonly commandStartX: number;
    readonly lastPromptLine: string | undefined;
    readonly continuationPrompt: string | undefined;
    readonly lastUserInput: string;
}

export type { IPromptInputModel, IPromptInputModelState, ISerializedPromptInputModel };
