import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { DataTransfers } from '../../../../base/browser/dnd.js';
import { DragAndDropObserver, $ } from '../../../../base/browser/dom.js';
import { renderLabelWithIcons } from '../../../../base/browser/ui/iconLabel/iconLabels.js';
import { coalesce } from '../../../../base/common/arrays.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { basename } from '../../../../base/common/resources.js';
import { localize } from '../../../../nls.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.service.js';
import { containsDragType, extractEditorsDropData } from '../../../../platform/dnd/browser/dnd.js';
import { Themable } from '../../../../platform/theme/common/themeService.js';
import { IThemeService } from '../../../../platform/theme/common/themeService.service.js';
var ChatDragAndDropType;
( ((function(ChatDragAndDropType) {
    ChatDragAndDropType[ChatDragAndDropType["FILE"] = 0] = "FILE";
    ChatDragAndDropType[ChatDragAndDropType["IMAGE"] = 1] = "IMAGE";
})(ChatDragAndDropType || (ChatDragAndDropType = {}))));
let ChatDragAndDrop = class ChatDragAndDrop extends Themable {
    constructor(contianer, inputPart, styles, themeService, configurationService) {
        super(themeService);
        this.contianer = contianer;
        this.inputPart = inputPart;
        this.styles = styles;
        this.configurationService = configurationService;
        this.overlayTextBackground = '';
        let mouseInside = false;
        this._register(( (new DragAndDropObserver(this.contianer, {
            onDragEnter: (e) => {
                if (!mouseInside) {
                    mouseInside = true;
                    this.onDragEnter(e);
                }
            },
            onDragOver: (e) => {
                e.stopPropagation();
            },
            onDragLeave: (e) => {
                this.onDragLeave(e);
                mouseInside = false;
            },
            onDrop: (e) => {
                this.onDrop(e);
                mouseInside = false;
            },
        }))));
        this.overlay = document.createElement('div');
        this.overlay.classList.add('chat-dnd-overlay');
        this.contianer.appendChild(this.overlay);
        this.updateStyles();
    }
    onDragEnter(e) {
        const estimatedDropType = this.guessDropType(e);
        if (estimatedDropType !== undefined) {
            e.stopPropagation();
            e.preventDefault();
        }
        this.updateDropFeedback(e, estimatedDropType);
    }
    onDragLeave(e) {
        this.updateDropFeedback(e, undefined);
    }
    onDrop(e) {
        this.updateDropFeedback(e, undefined);
        const contexts = this.getAttachContext(e);
        if (contexts.length === 0) {
            return;
        }
        e.stopPropagation();
        e.preventDefault();
        const currentContextIds = ( (new Set(
             (Array.from(this.inputPart.attachedContext).map(context => context.id))
        )));
        const filteredContext = [];
        for (const context of contexts) {
            if (!( (currentContextIds.has(context.id)))) {
                currentContextIds.add(context.id);
                filteredContext.push(context);
            }
        }
        this.inputPart.attachContext(false, ...filteredContext);
    }
    updateDropFeedback(e, dropType) {
        const showOverlay = dropType !== undefined;
        if (e.dataTransfer) {
            e.dataTransfer.dropEffect = showOverlay ? 'copy' : 'none';
        }
        this.setOverlay(dropType);
    }
    isImageDnd(e) {
        if (containsDragType(e, 'image')) {
            return true;
        }
        if (containsDragType(e, DataTransfers.FILES)) {
            const files = e.dataTransfer?.files;
            if (files && files.length > 0) {
                const file = files[0];
                return file.type.startsWith('image/');
            }
            const items = e.dataTransfer?.items;
            if (items && items.length > 0) {
                const item = items[0];
                return item.type.startsWith('image/');
            }
        }
        return false;
    }
    guessDropType(e) {
        if (this.isImageDnd(e)) {
            const imageDndSupported = this.configurationService.getValue('chat.experimental.imageAttachments');
            return imageDndSupported ? ChatDragAndDropType.IMAGE : undefined;
        }
        else if (containsDragType(e, DataTransfers.FILES, DataTransfers.INTERNAL_URI_LIST)) {
            return ChatDragAndDropType.FILE;
        }
        return undefined;
    }
    isDragEventSupported(e) {
        const dropType = this.guessDropType(e);
        return dropType !== undefined;
    }
    getDropTypeName(type) {
        switch (type) {
            case ChatDragAndDropType.FILE: return ( localize(7196, 'File'));
            case ChatDragAndDropType.IMAGE: return ( localize(7197, 'Image'));
        }
    }
    getAttachContext(e) {
        if (!this.isDragEventSupported(e)) {
            return [];
        }
        const data = extractEditorsDropData(e);
        return coalesce(( (data.map(editorInput => {
            return this.resolveAttachContext(editorInput);
        }))));
    }
    resolveAttachContext(editorInput) {
        const imageContext = getImageAttachContext(editorInput);
        if (imageContext) {
            const isImageDndSupported = this.configurationService.getValue('chat.experimental.imageAttachments');
            return isImageDndSupported ? imageContext : undefined;
        }
        return getEditorAttachContext(editorInput);
    }
    setOverlay(type) {
        this.overlayText?.remove();
        this.overlayText = undefined;
        if (type !== undefined) {
            const typeName = this.getDropTypeName(type);
            const iconAndtextElements = renderLabelWithIcons(`$(${Codicon.attach.id}) ${( localize(7198, 'Attach {0} as Context', typeName))}`);
            const htmlElements = ( (iconAndtextElements.map(element => {
                if (typeof element === 'string') {
                    return $('span.overlay-text', undefined, element);
                }
                return element;
            })));
            this.overlayText = $('span.attach-context-overlay-text', undefined, ...htmlElements);
            this.overlayText.style.backgroundColor = this.overlayTextBackground;
            this.overlay.appendChild(this.overlayText);
        }
        this.overlay.classList.toggle('visible', type !== undefined);
    }
    updateStyles() {
        this.overlay.style.backgroundColor = this.getColor(this.styles.overlayBackground) || '';
        this.overlay.style.color = this.getColor(this.styles.listForeground) || '';
        this.overlayTextBackground = this.getColor(this.styles.listBackground) || '';
    }
};
ChatDragAndDrop = ( (__decorate([
    ( (__param(3, IThemeService))),
    ( (__param(4, IConfigurationService)))
], ChatDragAndDrop)));
function getEditorAttachContext(editor) {
    if (!editor.resource) {
        return undefined;
    }
    return getFileAttachContext(editor.resource);
}
function getFileAttachContext(resource) {
    return {
        value: resource,
        id: ( (resource.toString())),
        name: basename(resource),
        isFile: true,
        isDynamic: true
    };
}
function getImageAttachContext(editor) {
    if (!editor.resource) {
        return undefined;
    }
    if (/\.(png|jpg|jpeg|bmp|gif|tiff)$/i.test(editor.resource.path)) {
        const fileName = basename(editor.resource);
        return {
            id: ( (editor.resource.toString())),
            name: fileName,
            fullName: editor.resource.path,
            value: editor.resource,
            icon: Codicon.fileMedia,
            isDynamic: true,
            isImage: true,
            isFile: false
        };
    }
    return undefined;
}
export { ChatDragAndDrop };
