import { localize } from '../../../../nls.js';
import { RawContextKey } from '../../../../platform/contextkey/common/contextkey.js';
import { language } from '../../../../base/common/platform.js';
const HasSpeechProvider = ( (new RawContextKey('hasSpeechProvider', false, { type: 'boolean', description: ( localize(6329, "A speech provider is registered to the speech service.")) })));
const SpeechToTextInProgress = ( (new RawContextKey(
    'speechToTextInProgress',
    false,
    { type: 'boolean', description: ( localize(6330, "A speech-to-text session is in progress.")) }
)));
const TextToSpeechInProgress = ( (new RawContextKey(
    'textToSpeechInProgress',
    false,
    { type: 'boolean', description: ( localize(6331, "A text-to-speech session is in progress.")) }
)));
var SpeechToTextStatus;
( ((function(SpeechToTextStatus) {
    SpeechToTextStatus[SpeechToTextStatus["Started"] = 1] = "Started";
    SpeechToTextStatus[SpeechToTextStatus["Recognizing"] = 2] = "Recognizing";
    SpeechToTextStatus[SpeechToTextStatus["Recognized"] = 3] = "Recognized";
    SpeechToTextStatus[SpeechToTextStatus["Stopped"] = 4] = "Stopped";
    SpeechToTextStatus[SpeechToTextStatus["Error"] = 5] = "Error";
})(SpeechToTextStatus || (SpeechToTextStatus = {}))));
var TextToSpeechStatus;
( ((function(TextToSpeechStatus) {
    TextToSpeechStatus[TextToSpeechStatus["Started"] = 1] = "Started";
    TextToSpeechStatus[TextToSpeechStatus["Stopped"] = 2] = "Stopped";
    TextToSpeechStatus[TextToSpeechStatus["Error"] = 3] = "Error";
})(TextToSpeechStatus || (TextToSpeechStatus = {}))));
var KeywordRecognitionStatus;
( ((function(KeywordRecognitionStatus) {
    KeywordRecognitionStatus[KeywordRecognitionStatus["Recognized"] = 1] = "Recognized";
    KeywordRecognitionStatus[KeywordRecognitionStatus["Stopped"] = 2] = "Stopped";
    KeywordRecognitionStatus[KeywordRecognitionStatus["Canceled"] = 3] = "Canceled";
})(KeywordRecognitionStatus || (KeywordRecognitionStatus = {}))));
const SPEECH_LANGUAGE_CONFIG = "accessibility.voice.speechLanguage" ;
const SPEECH_LANGUAGES = {
    ['da-DK']: {
        name: ( localize(6332, "Danish (Denmark)"))
    },
    ['de-DE']: {
        name: ( localize(6333, "German (Germany)"))
    },
    ['en-AU']: {
        name: ( localize(6334, "English (Australia)"))
    },
    ['en-CA']: {
        name: ( localize(6335, "English (Canada)"))
    },
    ['en-GB']: {
        name: ( localize(6336, "English (United Kingdom)"))
    },
    ['en-IE']: {
        name: ( localize(6337, "English (Ireland)"))
    },
    ['en-IN']: {
        name: ( localize(6338, "English (India)"))
    },
    ['en-NZ']: {
        name: ( localize(6339, "English (New Zealand)"))
    },
    ['en-US']: {
        name: ( localize(6340, "English (United States)"))
    },
    ['es-ES']: {
        name: ( localize(6341, "Spanish (Spain)"))
    },
    ['es-MX']: {
        name: ( localize(6342, "Spanish (Mexico)"))
    },
    ['fr-CA']: {
        name: ( localize(6343, "French (Canada)"))
    },
    ['fr-FR']: {
        name: ( localize(6344, "French (France)"))
    },
    ['hi-IN']: {
        name: ( localize(6345, "Hindi (India)"))
    },
    ['it-IT']: {
        name: ( localize(6346, "Italian (Italy)"))
    },
    ['ja-JP']: {
        name: ( localize(6347, "Japanese (Japan)"))
    },
    ['ko-KR']: {
        name: ( localize(6348, "Korean (South Korea)"))
    },
    ['nl-NL']: {
        name: ( localize(6349, "Dutch (Netherlands)"))
    },
    ['pt-PT']: {
        name: ( localize(6350, "Portuguese (Portugal)"))
    },
    ['pt-BR']: {
        name: ( localize(6351, "Portuguese (Brazil)"))
    },
    ['ru-RU']: {
        name: ( localize(6352, "Russian (Russia)"))
    },
    ['sv-SE']: {
        name: ( localize(6353, "Swedish (Sweden)"))
    },
    ['tr-TR']: {
        name: ( localize(6354, "Turkish (Türkiye)"))
    },
    ['zh-CN']: {
        name: ( localize(6355, "Chinese (Simplified, China)"))
    },
    ['zh-HK']: {
        name: ( localize(6356, "Chinese (Traditional, Hong Kong)"))
    },
    ['zh-TW']: {
        name: ( localize(6357, "Chinese (Traditional, Taiwan)"))
    }
};
function speechLanguageConfigToLanguage(config, lang = language) {
    if (typeof config === 'string') {
        if (config === 'auto') {
            if (lang !== 'en') {
                const langParts = lang.split('-');
                return speechLanguageConfigToLanguage(`${langParts[0]}-${(langParts[1] ?? langParts[0]).toUpperCase()}`);
            }
        }
        else {
            if (SPEECH_LANGUAGES[config]) {
                return config;
            }
        }
    }
    return 'en-US';
}
export { HasSpeechProvider, KeywordRecognitionStatus, SPEECH_LANGUAGES, SPEECH_LANGUAGE_CONFIG, SpeechToTextInProgress, SpeechToTextStatus, TextToSpeechInProgress, TextToSpeechStatus, speechLanguageConfigToLanguage };
